@@ -148,7 +148,13 @@ class LexerTest(TemplateTest):
         """
         assert_raises(exceptions.CompileException, Lexer(template).parse)
 
-    def test_tag_many_quotes(self):
+    def test_closing_tag_many_spaces(self):
+        """test #367"""
+        template = '<%def name="foo()"> this is a def. </%' + " " * 10000
+        assert_raises(exceptions.SyntaxException, Lexer(template).parse)
+
+    def test_opening_tag_many_quotes(self):
+        """test #366"""
         template = "<%0" + '"' * 3000
         assert_raises(exceptions.SyntaxException, Lexer(template).parse)
 
@@ -195,8 +201,8 @@ class LexerTest(TemplateTest):
                 {},
                 [
                     Text("""\n\n""", (1, 1)),
-                    Text("""% some whatever.\n\n""", (3, 2)),
-                    Text("   %% more some whatever\n", (5, 2)),
+                    Text("""% some whatever.\n\n    """, (3, 2)),
+                    Text("% more some whatever\n", (5, 6)),
                     ControlLine("if", "if foo:", False, (6, 1)),
                     ControlLine("if", "endif", True, (7, 1)),
                     Text("        ", (8, 1)),
@@ -204,6 +210,50 @@ class LexerTest(TemplateTest):
             ),
         )
 
+    def test_percent_escape2(self):
+        template = """%% do something
+%%% do something
+if <some condition>:
+    %%%% do something
+        """
+        node = Lexer(template).parse()
+        self._compare(
+            node,
+            TemplateNode(
+                {},
+                [
+                    Text("% do something\n", (1, 2)),
+                    Text(
+                        "%% do something\nif <some condition>:\n    ", (2, 2)
+                    ),
+                    Text("%%% do something\n        ", (4, 6)),
+                ],
+            ),
+        )
+
+    def test_percent_escape_with_control_block(self):
+        template = """
+% for i in [1, 2, 3]:
+    %% do something ${i}
+% endfor
+"""
+        node = Lexer(template).parse()
+        self._compare(
+            node,
+            TemplateNode(
+                {},
+                [
+                    Text("\n", (1, 1)),
+                    ControlLine("for", "for i in [1, 2, 3]:", False, (2, 1)),
+                    Text("    ", (3, 1)),
+                    Text("% do something ", (3, 6)),
+                    Expression("i", [], (3, 21)),
+                    Text("\n", (3, 25)),
+                    ControlLine("for", "endfor", True, (4, 1)),
+                ],
+            ),
+        )
+
     def test_old_multiline_comment(self):
         template = """#*"""
         node = Lexer(template).parse()
